@@ -1,5 +1,4 @@
 from collections.abc import Callable, Awaitable
-from typing import Type
 
 from bolinette import blnt, core, web, BolinetteExtension
 from bolinette.blnt.commands import Command
@@ -7,12 +6,12 @@ from bolinette.blnt.commands import Command
 
 class BolinetteCache:
     def __init__(self):
-        self.models: dict[str, Type['core.Model']] = {}
-        self.mixins: dict[str, Type['core.Mixin']] = {}
-        self.services: dict[str, Type['core.Service']] = {}
-        self.controllers: dict[str, Type['web.Controller']] = {}
-        self.middlewares: dict[str, Type['web.Middleware']] = {}
-        self.topics: dict[str, Type['web.Topic']] = {}
+        self.models: dict[str, type[core.Model]] = {}
+        self.mixins: dict[str, type[core.Mixin]] = {}
+        self.services: dict[str, type[core.Service | core.SimpleService]] = {}
+        self.controllers: dict[str, type[web.Controller]] = {}
+        self.middlewares: dict[str, type[web.Middleware]] = {}
+        self.topics: dict[str, type[web.Topic]] = {}
         self.init_funcs: list[InitFunc] = []
         self.commands: dict[str, Command] = {}
         self.seeders = []
