@@ -1,16 +1,30 @@
 from typing import Any
 
-from bolinette import blnt, core
+from bolinette import abc, blnt
 from bolinette.blnt.objects import PaginationParams, OrderByParams
+from bolinette.decorators import injected
 from bolinette.exceptions import EntityNotFoundError
 
 
-class Service:
+class SimpleService(abc.WithContext):
     __blnt__: 'ServiceMetadata' = None
 
     def __init__(self, context: 'blnt.BolinetteContext'):
-        self.context = context
-        self.repo: core.Repository = context.repo(self.__blnt__.model_name)
+        super().__init__(context)
+
+    def __repr__(self):
+        return f'<Service {self.__blnt__.name}>'
+
+
+class Service(SimpleService):
+    __blnt__: 'ServiceMetadata' = None
+
+    def __init__(self, context: 'blnt.BolinetteContext'):
+        super().__init__(context)
+
+    @injected
+    def repo(self, inject: 'blnt.BolinetteInjection'):
+        return inject.repositories.require(self.__blnt__.model_name)
 
     def __repr__(self):
         return f'<Service {self.__blnt__.name}>'
@@ -54,16 +68,6 @@ class Service:
         return await self.repo.delete(entity, **kwargs)
 
 
-class SimpleService:
-    __blnt__: 'ServiceMetadata' = None
-
-    def __init__(self, context: 'blnt.BolinetteContext'):
-        self.context = context
-
-    def __repr__(self):
-        return f'<Service {self.__blnt__.name}>'
-
-
 class ServiceMetadata:
     def __init__(self, name: str, model_name: str):
         self.name = name
