@@ -6,8 +6,10 @@ from bolinette.decorators import service
 
 @service('tz')
 class TimezoneService(core.SimpleService):
-    async def get_all(self):
+    @staticmethod
+    async def get_all():
         return [tz for tz in pytz.all_timezones]
 
-    async def is_valid(self, key):
+    @staticmethod
+    async def is_valid(key):
         return key in pytz.all_timezones
