@@ -4,37 +4,58 @@ from sqlalchemy import orm as sqlalchemy_orm
 from bolinette import blnt, core, web, Extensions
 from bolinette.decorators import init_func
 from bolinette.exceptions import InitError
-from bolinette.utils import InitProxy
 
 
 @init_func(extension=Extensions.MODELS)
 async def init_model_classes(context: blnt.BolinetteContext):
-    models = {}
-    proxies = {}
+    def _init_model(model: core.Model):
+        def _init_column(_name: str, _attr: blnt.InstantiableAttribute[core.models.Column]):
+            return _attr.instantiate(name=_name, model=model)
 
-    # Instantiate models & process columns
-    for model_name, model_cls in blnt.cache.models.items():
-        db_key = model_cls.__blnt__.database
-        if db_key not in context.db:
-            raise InitError(f'Undefined "{db_key}" database for model "{model_name}"')
-        model = model_cls(context.db[db_key])
-        # Instantiate columns
-        for col_name, proxy in model.__props__.get_proxies(core.models.Column):
-            col = proxy.instantiate(name=col_name, model=model)
-            proxies[proxy] = col
-            setattr(model, col_name, col)
-        # Add mixin columns
+        def _init_relationship(_name: str, _attr: blnt.InstantiableAttribute[core.models.Relationship]):
+            _target_name = _attr.pop('model_name')
+            _target_model = context.inject.models.require(_target_name, immediate=True)
+            _fk_name = _attr.pop('foreign_key')
+            _foreign_key = None
+            if _fk_name is not None:
+                _foreign_key = getattr(model, _fk_name, None)
+            _remote_name = _attr.pop('remote_side')
+            _remote_side = None
+            _secondary_name = _attr.pop('secondary')
+            _secondary = None
+            if _secondary_name is not None:
+                _secondary = context.inject.models.require(_secondary_name, immediate=True)
+            if _remote_name is not None:
+                _remote_side = getattr(model, _remote_name)
+            return _attr.instantiate(name=_name, model=model, target_model=_target_model, secondary=_secondary,
+                                     foreign_key=_foreign_key, remote_side=_remote_side)
+
+        def _init_reference(_col: core.models.Column, _attr: blnt.InstantiableAttribute[core.models.Reference]):
+            _target_name = _attr.pop('model_name')
+            _target_model = context.inject.models.require(_target_name, immediate=True)
+            _target_col_name = _attr.pop('column_name')
+            _target_column = getattr(_target_model, _target_col_name, None)
+            if _target_column is None:
+                raise InitError(f'{model.__blnt__.name}.{_col.name}: '
+                                f'no "{_target_col_name}" column in "{_target_name}" model')
+            return _attr.instantiate(model=model, column=_col,
+                                     target_model=_target_model, target_column=_target_column)
+
+        # Instantiate mixins
         for mixin_name in model.__blnt__.mixins:
             if mixin_name not in blnt.cache.mixins:
                 raise InitError(f'Model "{model_name}": mixin "{mixin_name}" is not defined')
-            mixin = blnt.cache.mixins[mixin_name]()
-            for col_name, proxy in mixin.columns().items():
-                col = proxy.instantiate(name=col_name, model=model)
-                setattr(model, col_name, col)
-            model.__props__.mixins[mixin_name] = mixin
-        models[model_name] = model
+            model.__props__.mixins[mixin_name] = blnt.cache.mixins[mixin_name]()
+
+        # Instantiate columns
+        for col_name, attr_col in model.__props__.get_instantiable(core.models.Column):
+            _init_column(col_name, attr_col)
+            setattr(model, col_name, _init_column(col_name, attr_col))
+        # Add mixin columns
+        for _, mixin in model.__props__.mixins.items():
+            for col_name, attr_col in mixin.columns().items():
+                setattr(model, col_name, _init_column(col_name, attr_col))
 
-    for model_name, model in models.items():
         # Process auto generated primary keys
         primary = [c for _, c in model.__props__.get_columns() if c.primary_key]
         if not primary:
@@ -52,57 +73,41 @@ async def init_model_classes(context: blnt.BolinetteContext):
         else:
             model.__props__.entity_key = entity_key
         if model.__props__.entity_key is None:
-            raise InitError(f'No entity key defined for "{model_name}" model. '
+            raise InitError(f'No entity key defined for "{model.__blnt__.name}" model. '
                             'Mark one column or more as entity_key=True.')
 
-    # Process relationships
-    for model_name, model in models.items():
-        # Instantiate relationships
-        for rel_name, proxy in model.__props__.get_proxies(core.models.Relationship):
-            rel = proxy.instantiate(name=rel_name, model=model, models=models)
-            proxies[proxy] = rel
-            setattr(model, rel_name, rel)
+        # Process relationships
+        for rel_name, attr_rel in model.__props__.get_instantiable(core.models.Relationship):
+            setattr(model, rel_name, _init_relationship(rel_name, attr_rel))
         # Add mixin relationships
         for _, mixin in model.__props__.mixins.items():
-            for rel_name, proxy in mixin.relationships(model).items():
-                rel = proxy.instantiate(name=rel_name, model=model, models=models)
-                setattr(model, rel_name, rel)
+            for rel_name, attr_rel in mixin.relationships().items():
+                setattr(model, rel_name, _init_relationship(rel_name, attr_rel))
 
-    # Process references
-    for model_name, model in models.items():
         # Instantiate references
         for col_name, col in model.__props__.get_columns():
-            if isinstance(col.reference, InitProxy) and col.reference.of_type(core.models.Reference):
-                col.reference = col.reference.instantiate(model=model, column=col, models=models)
-        # Process back references
+            if isinstance(col.reference, blnt.InstantiableAttribute):
+                col.reference = _init_reference(col, col.reference)
+        # Instantiate back references
         added_back_refs: dict[core.Model, list[core.models.ColumnList]] = {}
         for rel_name, rel in model.__props__.get_relationships():
-            # Instantiate back references
-            if isinstance(rel.backref, InitProxy) and rel.backref.of_type(core.models.Backref):
+            if isinstance(rel.backref, blnt.InstantiableAttribute):
                 rel.backref = rel.backref.instantiate(model=model, relationship=rel)
                 if rel.backref.key not in added_back_refs:
                     added_back_refs[rel.target_model] = []
                 added_back_refs[rel.target_model].append(
                     core.models.ColumnList(rel.backref.key, rel.target_model, model))
-            # Link foreign keys
-            if isinstance(rel.foreign_key, InitProxy) and rel.foreign_key.of_type(core.models.Column):
-                rel.foreign_key = proxies[rel.foreign_key]
-            # Link remote side foreign keys for same-model-references
-            if isinstance(rel.remote_side, InitProxy) and rel.remote_side.of_type(core.models.Column):
-                rel.remote_side = proxies[rel.remote_side]
-        # Add back refs to the target model
-        for target_model, back_refs in added_back_refs.items():
-            for back_ref in back_refs:
-                setattr(target_model, back_ref.name, back_ref)
 
-    for model_name, model in models.items():
-        context.add_model(model_name, model)
+    context.inject.__add_collection__('models', core.Model)
+    for model_name, model_cls in blnt.cache.models.items():
+        context.inject.models.__add_type__(model_name, model_cls, func=_init_model)
 
 
 @init_func(extension=Extensions.MODELS)
 async def init_relational_models(context: blnt.BolinetteContext):
     models = {}
-    for model_name, model in context.models:
+    for model_name in context.inject.models.registered():
+        model = context.inject.models.require(model_name, immediate=True)
         if model.__props__.database.relational:
             models[model_name] = model
     orm_tables = {}
@@ -110,7 +115,6 @@ async def init_relational_models(context: blnt.BolinetteContext):
     for model_name, model in models.items():
         orm_cols[model_name] = {}
         for att_name, attribute in model.__props__.get_columns():
-            attribute.name = att_name
             ref = None
             if attribute.reference:
                 ref = sqlalchemy.ForeignKey(attribute.reference.target_path)
@@ -144,7 +148,7 @@ async def init_relational_models(context: blnt.BolinetteContext):
         for att_name, attribute in model.__props__.get_properties():
             setattr(orm_model, att_name, property(attribute.function))
 
-        context.add_table(model_name, orm_model)
+        model.__props__.database.add_table(model_name, orm_model)
 
 
 @init_func(extension=Extensions.MODELS)
@@ -154,36 +158,41 @@ async def init_databases(context: blnt.BolinetteContext):
 
 @init_func(extension=Extensions.MODELS)
 async def init_repositories(context: blnt.BolinetteContext):
-    for model_name, model in context.models:
-        repo = core.Repository(model_name, model, context)
-        model.__repo__ = repo
-        context.add_repo(model_name, repo)
+    def _init_repo(repo: core.Repository):
+        repo.model.__repo__ = repo
+
+    context.inject.__add_collection__('repositories', core.Repository)
+    for model_name in context.inject.models.registered():
+        model = context.inject.models.require(model_name, immediate=True)
+        context.inject.repositories.__add_type__(
+            model_name, core.Repository, params={'model': model, 'name': model_name}, func=_init_repo)
 
 
 @init_func(extension=Extensions.MODELS)
 async def init_mappings(context: blnt.BolinetteContext):
-    for model_name, model in context.models:
+    for model_name in context.inject.models.registered():
+        model = context.inject.models.require(model_name, immediate=True)
         context.mapper.register(model_name, model)
 
 
 @init_func(extension=Extensions.MODELS)
 async def init_services(context: blnt.BolinetteContext):
+    context.inject.__add_collection__('services', core.SimpleService)
     for service_name, service_cls in blnt.cache.services.items():
-        context.add_service(service_name, service_cls(context))
+        context.inject.services.__add_type__(service_name, service_cls)
 
 
 @init_func(extension=Extensions.WEB)
 async def init_controllers(context: blnt.BolinetteContext):
-    def _init_route(_ctrl: web.Controller, _proxy: InitProxy):
-        _route = _proxy.instantiate(controller=_ctrl)
-        if _route.inner_route is not None:
-            _route.inner_route, _route.func = _init_route(_ctrl, _route.inner_route)
-        return _route, _route.func
+    def _init_ctrl(controller: web.Controller):
+        def _init_route(_attr: blnt.InstantiableAttribute[web.ControllerRoute]):
+            _route = _attr.instantiate(controller=controller)
+            if _route.inner_route is not None:
+                _route.inner_route, _route.func = _init_route(_route.inner_route)  # type: ignore
+            return _route, _route.func
 
-    for controller_name, controller_cls in blnt.cache.controllers.items():
-        controller = controller_cls(context)
-        for route_name, proxy in controller.__props__.get_proxies(web.ControllerRoute):
-            route, _ = _init_route(controller, proxy)
+        for route_name, proxy in controller.__props__.get_instantiable(web.ControllerRoute):
+            route, _ = _init_route(proxy)
             setattr(controller, route_name, route)
         for _, route in controller.__props__.get_routes():
             route.controller = controller
@@ -191,13 +200,16 @@ async def init_controllers(context: blnt.BolinetteContext):
         for route in controller.default_routes():
             route.controller = controller
             route.setup()
-        context.add_controller(controller_name, controller)
+
+    context.inject.__add_collection__('controllers', web.Controller)
+    for controller_name, controller_cls in blnt.cache.controllers.items():
+        context.inject.controllers.__add_type__(controller_name, controller_cls, func=_init_ctrl)
 
 
-@init_func(extension=Extensions.SOCKETS)
-async def init_topics(context: blnt.BolinetteContext):
-    for topic_name, topic_cls in blnt.cache.topics.items():
-        topic = topic_cls(context)
-        context.sockets.add_topic(topic_name, topic)
-        for _, channel in topic.__props__.get_channels():
-            context.sockets.add_channel(topic_name, channel)
+# @init_func(extension=Extensions.SOCKETS)
+# async def init_topics(context: blnt.BolinetteContext):
+#     for topic_name, topic_cls in blnt.cache.topics.items():
+#         topic = topic_cls(context)
+#         context.sockets.add_topic(topic_name, topic)
+#         for _, channel in topic.__props__.get_channels():
+#             context.sockets.add_channel(topic_name, channel)
