@@ -1,27 +1,28 @@
 import re
 from collections.abc import Generator, Callable
-from typing import Any, Optional, Union
+from typing import Any
 
 from aiohttp.web_request import Request
 
-from bolinette import blnt, core, web
+from bolinette import abc, blnt, core, web
+from bolinette.decorators import injected
 from bolinette.exceptions import InitError
 from bolinette.utils import functions
 
 
-class Controller:
+class Controller(abc.WithContext):
     __blnt__: 'ControllerMetadata' = None
 
     def __init__(self, context: 'blnt.BolinetteContext'):
+        super().__init__(context)
         self.__props__ = ControllerProps(self)
-        self.context = context
         self.response = web.Response(context)
         if self.__blnt__.use_service:
             self.defaults = ControllerDefaults(self)
 
-    @property
-    def service(self) -> 'core.Service':
-        return self.context.service(self.__blnt__.service_name)
+    @injected
+    def service(self, inject: 'blnt.BolinetteInjection') -> 'core.Service':
+        return inject.services.require(self.__blnt__.service_name)
 
     def default_routes(self) -> list['web.ControllerRoute']:
         return []
@@ -51,7 +52,7 @@ class ControllerProps(blnt.Properties):
 
 class ControllerRoute:
     def __init__(self, controller: 'web.Controller', func: Callable, path: str, method: web.HttpMethod,
-                 docstring: Optional[str], expects: 'Expects' = None, returns: 'Returns' = None,
+                 docstring: str | None, expects: 'Expects' = None, returns: 'Returns' = None,
                  inner_route: 'ControllerRoute' = None, middlewares: list[str] = None):
         self.controller = controller
         self.func = func
@@ -179,15 +180,19 @@ class Returns:
         self.skip_none = skip_none
 
 
-class ControllerDefaults:
+class ControllerDefaults(abc.WithContext):
     PARAM_NAME_DOT_REGEX = re.compile(r'\.')
     PARAM_NAME_CHAR_REGEX = re.compile(r'[^a-zA-Z0-9_]')
 
     def __init__(self, controller: Controller):
+        super().__init__(controller.context)
         self.controller = controller
-        self.service: core.Service = controller.service
 
-    def _get_url_keys(self, key: Optional[Union[str, list[str]]], *, route: str) -> list[str]:
+    @injected
+    def service(self, inject: 'blnt.BolinetteInjection') -> 'core.Service':
+        return inject.services.require(self.controller.__blnt__.service_name)
+
+    def _get_url_keys(self, key: str | list[str] | None, *, route: str) -> list[str]:
         if key is None:
             entity_key = self.service.repo.model.__props__.entity_key
             if entity_key is None:
@@ -224,7 +229,7 @@ class ControllerDefaults:
                                returns=Returns(model_name, returns, as_list=True),
                                middlewares=middlewares)
 
-    def get_one(self, returns='default', *, key: Union[str, list[str], None] = None,
+    def get_one(self, returns='default', *, key: str | list[str] | None = None,
                 prefix='', middlewares: list[str] = None, docstring: str = None):
         model_name = self.service.__blnt__.model_name
         url_keys = self._get_url_keys(key, route='get_one')
