@@ -1,19 +1,18 @@
 import asyncio
 from json import JSONDecodeError
-from typing import Union
 
 import aiohttp
 from aiohttp import web as aio_web
 from aiohttp.web_request import Request
 
-from bolinette import blnt, web
+from bolinette import abc, blnt, web
 from bolinette.exceptions import APIError
 from bolinette.utils.functions import async_invoke
 
 
-class BolinetteSockets:
+class BolinetteSockets(abc.WithContext):
     def __init__(self, context: 'blnt.BolinetteContext'):
-        self.context = context
+        super().__init__(context)
         self._topics: dict[str, 'web.Topic'] = {}
         self._channels: dict[str, list['web.TopicChannel']] = {}
         self._socket_sessions: dict[str, aio_web.WebSocketResponse] = {}
@@ -33,7 +32,7 @@ class BolinetteSockets:
     def channels(self, topic: str) -> list['web.TopicChannel']:
         return self._channels.get(topic) or []
     
-    async def send_message(self, topic: str, channels: Union[str, list[str]], message):
+    async def send_message(self, topic: str, channels: str | list[str], message):
         socket_topic = self.topic(topic)
         if socket_topic is None:
             return
@@ -75,7 +74,7 @@ class SocketHandler:
 
     async def __call__(self, request: Request):
         context: blnt.BolinetteContext = request.app['blnt']
-        user_service = context.service('user')
+        user_service = context.inject.services.require('user', immediate=True)
         socket = aio_web.WebSocketResponse()
         await socket.prepare(request)
 
